@@ -92,7 +92,7 @@ func (w *HTMLWriter) WriterWithExtensions() Writer {
 func (w *HTMLWriter) Before(d *Document) {
 	w.document = d
 	w.log = d.Log
-	if title := d.Get("TITLE"); title != "" {
+	if title := d.Get("TITLE"); title != "" && w.document.GetOption("title") != "nil" {
 		titleDocument := d.Parse(strings.NewReader(title), d.Path)
 		if titleDocument.Error == nil {
 			title = w.WriteNodesAsString(titleDocument.Nodes...)
@@ -249,6 +249,7 @@ func (w *HTMLWriter) WriteHeadline(h Headline) {
 		}
 	}
 
+	w.WriteString(fmt.Sprintf(`<div id="outline-container-%s" class="outline-%d">`, h.ID(), h.Lvl+1) + "\n")
 	w.WriteString(fmt.Sprintf(`<h%d id="%s">`, h.Lvl+1, h.ID()) + "\n")
 	if w.document.GetOption("todo") != "nil" && h.Status != "" {
 		w.WriteString(fmt.Sprintf(`<span class="todo">%s</span>`, h.Status) + "\n")
@@ -267,7 +268,10 @@ func (w *HTMLWriter) WriteHeadline(h Headline) {
 		w.WriteString(fmt.Sprintf(`<span class="tags">%s</span>`, strings.Join(tags, "&#xa0;")))
 	}
 	w.WriteString(fmt.Sprintf("\n</h%d>\n", h.Lvl+1))
-	WriteNodes(w, h.Children...)
+	if content := w.WriteNodesAsString(h.Children...); content != "" {
+		w.WriteString(fmt.Sprintf(`<div id="outline-text-%s" class="outline-text-%d">`, h.ID(), h.Lvl+1) + "\n" + content + "</div>\n")
+	}
+	w.WriteString("</div>\n")
 }
 
 func (w *HTMLWriter) WriteText(t Text) {
@@ -339,22 +343,45 @@ func (w *HTMLWriter) WriteRegularLink(l RegularLink) {
 		url = url[len("file:"):]
 	}
 	if prefix := w.document.Links[l.Protocol]; prefix != "" {
-		url = html.EscapeString(prefix) + url[len(l.Protocol)+1:]
-	}
-	description := url
-	if l.Description != nil {
-		description = w.WriteNodesAsString(l.Description...)
+		url = html.EscapeString(prefix) + strings.TrimPrefix(url, l.Protocol+":")
 	}
 	switch l.Kind() {
 	case "image":
-		w.WriteString(fmt.Sprintf(`<img src="%s" alt="%s" title="%s" />`, url, description, description))
+		if l.Description == nil {
+			w.WriteString(fmt.Sprintf(`<img src="%s" alt="%s" title="%s" />`, url, url, url))
+		} else {
+			description := strings.TrimPrefix(String(l.Description), "file:")
+			w.WriteString(fmt.Sprintf(`<a href="%s"><img src="%s" alt="%s" /></a>`, url, description, description))
+		}
 	case "video":
-		w.WriteString(fmt.Sprintf(`<video src="%s" title="%s">%s</video>`, url, description, description))
+		if l.Description == nil {
+			w.WriteString(fmt.Sprintf(`<video src="%s" title="%s">%s</video>`, url, url, url))
+		} else {
+			description := strings.TrimPrefix(String(l.Description), "file:")
+			w.WriteString(fmt.Sprintf(`<a href="%s"><video src="%s" title="%s"></video></a>`, url, description, description))
+		}
 	default:
+		description := url
+		if l.Description != nil {
+			description = w.WriteNodesAsString(l.Description...)
+		}
 		w.WriteString(fmt.Sprintf(`<a href="%s">%s</a>`, url, description))
 	}
 }
 
+func (w *HTMLWriter) WriteMacro(m Macro) {
+	if macro := w.document.Macros[m.Name]; macro != "" {
+		for i, param := range m.Parameters {
+			macro = strings.Replace(macro, fmt.Sprintf("$%d", i+1), param, -1)
+		}
+		macroDocument := w.document.Parse(strings.NewReader(macro), w.document.Path)
+		if macroDocument.Error != nil {
+			w.log.Printf("bad macro: %s -> %s: %v", m.Name, macro, macroDocument.Error)
+		}
+		WriteNodes(w, macroDocument.Nodes...)
+	}
+}
+
 func (w *HTMLWriter) WriteList(l List) {
 	tags, ok := listTags[l.Kind]
 	if !ok {
@@ -398,11 +425,8 @@ func (w *HTMLWriter) WriteParagraph(p Paragraph) {
 		return
 	}
 	w.WriteString("<p>")
-	if _, ok := p.Children[0].(LineBreak); !ok {
-		w.WriteString("\n")
-	}
 	WriteNodes(w, p.Children...)
-	w.WriteString("\n</p>\n")
+	w.WriteString("</p>\n")
 }
 
 func (w *HTMLWriter) WriteExample(e Example) {
@@ -449,23 +473,31 @@ func (w *HTMLWriter) WriteNodeWithName(n NodeWithName) {
 
 func (w *HTMLWriter) WriteTable(t Table) {
 	w.WriteString("<table>\n")
-	beforeFirstContentRow := true
+	inHead := len(t.SeparatorIndices) > 0 &&
+		t.SeparatorIndices[0] != len(t.Rows)-1 &&
+		(t.SeparatorIndices[0] != 0 || len(t.SeparatorIndices) > 1 && t.SeparatorIndices[len(t.SeparatorIndices)-1] != len(t.Rows)-1)
+	if inHead {
+		w.WriteString("<thead>\n")
+	} else {
+		w.WriteString("<tbody>\n")
+	}
 	for i, row := range t.Rows {
-		if row.IsSpecial || len(row.Columns) == 0 {
-			continue
-		}
-		if beforeFirstContentRow {
-			beforeFirstContentRow = false
-			if i+1 < len(t.Rows) && len(t.Rows[i+1].Columns) == 0 {
-				w.WriteString("<thead>\n")
-				w.writeTableColumns(row.Columns, "th")
+		if len(row.Columns) == 0 && i != 0 && i != len(t.Rows)-1 {
+			if inHead {
 				w.WriteString("</thead>\n<tbody>\n")
-				continue
+				inHead = false
 			} else {
-				w.WriteString("<tbody>\n")
+				w.WriteString("</tbody>\n<tbody>\n")
 			}
 		}
-		w.writeTableColumns(row.Columns, "td")
+		if row.IsSpecial {
+			continue
+		}
+		if inHead {
+			w.writeTableColumns(row.Columns, "th")
+		} else {
+			w.writeTableColumns(row.Columns, "td")
+		}
 	}
 	w.WriteString("</tbody>\n</table>\n")
 }
