@@ -67,6 +67,11 @@ func (d *Document) parseKeyword(i int, stop stopFn) (int, Node) {
 			d.Links[parts[0]] = parts[1]
 		}
 		return 1, k
+	case "MACRO":
+		if parts := strings.Split(k.Value, " "); len(parts) >= 2 {
+			d.Macros[parts[0]] = parts[1]
+		}
+		return 1, k
 	case "CAPTION", "ATTR_HTML":
 		consumed, node := d.parseAffiliated(i, stop)
 		if consumed != 0 {
